SITE_WITH_DOMAIN_DOES_NOT_EXIST = 'site_with_domain_does_not_exist'
INCORRECT_PHONE_NUMBER = 'incorrect_phone_number'
INCORRECT_CONFIRMATION_PIN = 'incorrect_confirmation_pin'
EMAIL_ALREADY_REGISTERED_ON_DOMAIN = 'email_already_registered_on_domain'
PHONE_NUMBER_ALREADY_REGISTERED_ON_DOMAIN = 'phone_number_already_registered_on_domain'
INVALID_PASSWORD = 'invalid_password'
CREDENTIAL_NOT_CONFIRMED = 'credential_not_confirmed'
USER_WITH_PROVIDED_CREDENTIALS_DOES_NOT_REGISTERED_ON_SPECIFIED_DOMAIN = 'user_with_provided_credentials_does_not_registered_on_specified_domain'
TOO_FREQUENT_SMS_REQUESTS = 'too_frequent_sms_requests'
